#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © Huawei Technologies Co., Ltd. 2022-2023. All rights reserved.
"""Mindpet sdk APIs."""

from mindpet import mindpet_sdk

__all__ = ["mindpet_sdk"]
