#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Copyright © Huawei Technologies Co., Ltd. 2022-2023. All rights reserved.
"""

from mindpet.log.log import get_logger, logger, operation_logger_without_std, operation_logger, logger_without_std

__all__ = ['get_logger', 'logger', 'logger_without_std', 'operation_logger', 'operation_logger_without_std']
