#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © Huawei Technologies Co., Ltd. 2022-2023. All rights reserved.
"""Constants for delta"""
from mindspore import dtype as mstype

# 算法精度与类型控制
VALID_TENSOR_DATATYPE = [mstype.float16, mstype.float32, mstype.bfloat16]
